function xyN=HZalign(xy,Hz,varargin)
%% aligns movements according Hz
%% required data format: timestamp,trace,x,y
%
% INPUT:    xy = data matrix containing movements
%                with columns: timestamp(ms),traceID,x-coordinates,y-coordinates.
%           Hz = frame rate to which the data shall be reduced or stretched and aligned.
%           varargin = whether to substitute NaN points with next point 0=no.
%
% OUTPUT:   xyN = data matrix containing Hz-aligned movements
%                 with columns: videoframe,x1,y1,x2,y2,...xn,yn. 
%
% RMF, 2019.

%% Check and set arguments.
narginchk(2,3); %check number of input arguments.
nargoutchk(1,1); %check number of output arguments.
if nargin==3
    varargin{1}=logical(varargin{1}*1); %set varargin to 1 if not 0.
else
    varargin{1}=true; %set varargin to true, if not 0.
end

%% Align traces according to Hz.
[xy,~]=sortrows(double(xy),1); %sort according to timestamps.
if max(xy(2:end,1)-xy(1:end-1,1))>1000*60
    warning('MovVis:pauses','The data contains pauses of more than 1 minute'); %warn that large pauses are included in data.
    str=input('Do you want to continue? y/n\n','s');
    if str=='n'
        beep; %play beep to signal error.
        error('MovVis:stop','Programm terminated by user');
    elseif str=='y'
    else
        beep; %play beep to signal error.
        error('MovVis:stop','Not a valid answer');
    end
end
xy(:,5)=ceil((xy(:,1)-xy(1,1)+1)/1000*Hz)+1; %add Hz video frame index.
traces=unique(xy(:,2)); %all event labels.
xyN=[1:max(xy(:,5));zeros(2*length(traces),max(xy(:,5)))]'; %initialize xyN.
for n=1:length(traces) %loop over all events
    lackFram=setdiff(xyN(:,1),unique(xy(xy(:,2)==traces(n),5))); %lacking frame data of current event.
    xy(end+1:end+length(lackFram),:)=...
        [nan(length(lackFram),1),zeros(length(lackFram),1)+traces(n),...
        nan(length(lackFram),2),lackFram];
        %inserting lacking frame data with NaNs.
    xyN(:,(n-1)*length(traces)+2:(n-1)*length(traces)+3)=...
        [accumarray(xy(xy(:,2)==traces(n),5),xy(xy(:,2)==traces(n),3),[],@nanmean,NaN),...
        accumarray(xy(xy(:,2)==traces(n),5),xy(xy(:,2)==traces(n),4),[],@nanmean,NaN)];
        %write mean locs per video frame in xyN.
    if varargin{1}
        while ~isempty(lackFram)
            lackFram=find(all(isnan(xyN(:,(n-1)*length(traces)+2:(n-1)*length(traces)+3)),2)); %indices of NaN frames.
            lackFram=lackFram(find(lackFram(:,1)-(1:size(lackFram,1))'~=0,1,'first'):end); %keep immediate lacking data empty.
            xyN(lackFram,(n-1)*length(traces)+2:(n-1)*length(traces)+3)=...
                xyN(lackFram-1,(n-1)*length(traces)+2:(n-1)*length(traces)+3);
                %substitute empty frames with previous frames.
        end    
    end
end

end