function varargout=MovVis(data,varargin)
%% MovVis
%% MovVis creates videos and animated gifs with movement animations in desired alignment over a chosen background image.
%
% INPUT:    REQUIRED:
%           data =     either: full path of file containing the movement data in the required format
%                      or: matlab variable containing the movement data in the required format
%                      with columns: timestamp, categoryID columns 1-n, unitID, eventID, x, y. 
%                      Only positive integers allowed for all inputs.
%                      The movements of all category x event combinations are presented in parallel.
%                      Unit can be used to align movements within separate sub-units,
%                      e.g. until acting on 1. target, until acting on 2. target.
%           OPTIONAL:
%           picpath    = full path of background image - default is empty = grey background.
%                        xy-coordinates are interpreted in pixel on picture, make sure pic has right size.
%                        if specified, visualizations will be saved to visualization sub-folder in the containing folder.
%                        if not specified, visualizations will be saved to visualization sub-folder in current folder.
%           speed      = visualization speed - default is 25.
%                        25 = real time in starts-aligned 25Hz video and gif.
%                        higher values for slow-motion, lower values for fast-forward.
%           vissize    = vector [width,height] defining size of visualization - default is as backgroundimage.
%           alignment  = type of alignment - default is 'sample'.
%                        'starts' = starts aligned, thus normal speed differences included.
%                        'ends' = ends also aligned according to the slowest movement (see MovVis_instructions.txt).
%                        'units' = starts and ends aligned within sub-units according to slowest movement (see MovVis_intructions.txt).
%                        'all' = all three alignments are generated.
%           prefix     = prefix of output file - default is 'MovVis'.
%           drawtrace  = whether to draw trace (1=yes, 0=no) - default is no.
%           missub   = whether to substitute missing data with next data point (1=yes, 0=no) - default is yes.
%           the following styles can be modified:
%           marker     = string of marker styles ('od*s^x>+', see doc plot) - default is all circle.
%           faceCol    = marker face colors (RGB defined by numbers between 0 and 1) - default is 0,0,0 = black.
%           edgeCol    = marker edge colors (RGB defined by numbers between 0 and 1) - default as faceCol.
%           traceCol   = trace colors (RGB defined by numbers between 0 and 1) - default as faceCol.
%           traceLine  = cell of lines styles for the trace (see doc plot) - default is all solid.
%           markerSize = marker sizes - default is 10.
%           !!styles are applied to the data from events over the category columns in decreasing order!!
%           !!example: 4subjects in column2 x 3trials in column3 x 2events in column5:
%           !!maker='od*s+x' mark as follows:
%           !!o for event1 of trial1 of sub1, d for event2 of trial1 of sub1,
%           !!* for event1 of trial2 of sub1, s for event2 of trial2 of sub1,
%           !!+ for event1 of trial3 of sub1, x for event2 of trial3 of sub1,
%           !!o for event1 of trial1 of sub2, d for event2 of trial1 of sub2, and so on
%           !!marker='so' marks each event individually - event1 with 's', event2 with 'o'.
%           !!marker='ssoodd' marks each trial individually - trial1 with 's', trial2 with 'o', trial3 with 'd'.
%           !!marker=[repmat('s',2*3,1),repmat('o',2*3,1),repmat('d',2*3,1)] marks each subject individually in the same order.
%	        !!marker=sort(repmat('sod',2*3,1)) marks each subject individually, but in the reverse order.
%
% OUTPUT:   video and gif file of animated movement paths and picture of end-configuration.
%           OPTIONAL:
%           varargout{1} = matrix array containing the input movement data.
%           varargout{2} = cell array of aligned data matrices 1.starts, 2.ends, 3.units.
%                          column format: frameID, x1, y1, ..., xn, yn
%                          same order as with styles (xy of all events first,
%                          then all events of 2nd category of last category columns, etc.
%
% RMF, 2019

%% Check and set arguments.
narginchk(1,14); %check number of input arguments.
nargoutchk(0,2); %check number of output arguments.
if isempty(strfind(version,'R')) %if Octave.
    warning('off','Octave:mixed-string-concat'); %suppress warning of num2str.
    warning('off','Octave:array-as-logical'); %suppress warning of all of array as logical.
    warning('off','Octave:language-extension'); %disable warning for language extension.
end
vars=strsplit('picpath speed vissize alignment prefix drawtrace missub marker faceCol edgeCol traceCol traceLine markerSize'); %all possible variables.
formats=strsplit('char double double char char double double char double double double cell double'); %required variable formats.
defaults=strsplit('[] 25 [] ''starts'' ''MovVis'' 0 1 ''o'' [0,0,0] faceCol faceCol {''-''} 10'); %variable defaults.
if nargin>1 %if more than one input argument given.
    for i=1:length(varargin) %loop over all input variables.
        if strcmp(char(class(varargin{i})),formats(strcmp(vars,inputname(i+1)))) %if input name and format is valid.
            eval([inputname(i+1),'=','varargin{i};']); %retrieve input argument.
        else
            beep; %play beep to signal error.
            error('MovVis:InvalidInput',['''',inputname(i+1),''' has invalid name or format\nplease check MovVis instructions']);
        end
        if length(eval(inputname(i+1)))~=1 &&...
                (strcmp(inputname(i+1),'speed') || strcmp(inputname(i+1),'drawtrace') || strcmp(inputname(i+1),'missub'))
                %if to many entries in speed, drawtrace, or missub.
            eval([inputname(i+1),'=',inputname(i+1),'(1);']); %keep only first entry.
            warning('MovVis:wrongLength',[inputname(i+1),' reduced to first entry']);
        end
        if strfind(inputname(i+1),'Col') %if color variable.
            if eval(['min(min(',inputname(i+1),'))<0']) %if negative color values.
                eval([inputname(i+1),'(',inputname(i+1),'<0)=0;']); %set negative color values to black.
                warning('MovVis:negCol',['negative ',inputname(i+1),' values were set to black']);
            end
            if eval(['max(max(',inputname(i+1),'))>1']) %if color values higher than 1.
                eval([inputname(i+1),'=floor(',inputname(i+1),'/255);']); %interpret color values as RGB.
                warning('MovVis:RGBcol',[inputname(i+1),' values were interpreted as RGB values']);
            end
            if length(eval(inputname(i+1)))<3 %if only a single number given.
                eval([inputname(i+1),'=repmat(',inputname(i+1),',1,3);']); %triple number to create color value.
                warning('MovVis:SingleColChannel',[inputname(i+1),' value was tripled to create color channels']);
            end
        end
    end
end
if ~exist('drawtrace','var') && ~isempty(cell2mat(strfind(who,'trace')))
    drawtrace=1; %draw trace, if any trace variable was defined.
end
for i=1:length(vars) %loop over all possible input variables.
    if ~exist(vars{i},'var')
        eval([vars{i},'=',defaults{i},';']); %set default for missing variables.
    end
end
if ~any(strcmp({'starts','ends','units','all'},alignment))
    beep; %play beep to signal error.
    error('MovVis:alignment','alignment method unknown'); %error, if alignment string not known.
end
styles={marker,faceCol,edgeCol,traceCol,traceLine,markerSize}; %styles cell.

%% Read data.
if ischar(data) %if data is string.
    fid=fopen(data); %open file.
    header=fgets(fid); %read first line.
    ncols=sum(~strcmp(strsplit(header),'')); %number of columns in file.
    if ~isnan(str2double(header(1))) %if first line was no header.
        frewind(fid) %set file back to first line.
    end
    xy=textscan(fid,repmat('%f',1,ncols)); %read movement data.
    fclose(fid); %close file.
    xy=[xy{:}]; %reconstruct movement data.
else
    xy=data; %rename movement data variable.
end
if size(xy,2)<6 %if too few columns in file.
    beep; %play beep to signal error.
    error('MovVis:tooFewColumns','Data has too few columns\nmin 6 columns needed\nsee instructions'); %display error.
end
if nargout>=1 %if first output was called.
    varargout{1}=xy; %set xy as first output.
end
xy=round(xy); %round all entries.
xy(:,1)=xy(:,1)-(min(xy(:,1))<1)*(min(xy(:,1))-1); %increase timestamps to positve integers if needed.
for n=2:(size(xy,2)-5) %loop over all categories.
    xy(:,n)=xy(:,n)-(min(xy(:,n))<1)*(min(xy(:,n))-1); %increase IDs to positve integers if needed.
end
if min(min(xy))<0
    xy(xy<0)=NaN; %delete negative coordinates from viewing.
%     xy(:,end-1)=xy(:,end-1)-min(min(xy(:,end-1))); %shift x to positive range.
%     xy(:,end)=xy(:,end)-min(min(xy(:,end))); %shift y to positive range.
    warning('MovVis:negData','negative data points were deleted'); %warning that negative points were deleted.
end
if any(any(isnan(xy(:,end-1:end)))) && missub==1 %if there are missing data points that will be substituted.
    warning('MovVis:missubDefault','Notice that missing data points will be substituted by previous points (missub=0).'); %dislay warning that missub is set to default.
end

%% Load required Ocatave packages.
if isempty(strfind(version,'R')) %if Octave.
    try
        pkg load statistics; %load statistics package.
    catch
        error('MovVis:Octave:Statistics','Statistics package for Octave not installed'); %display error that statistics package for Octave is not installe.  
    end
    try
        pkg load image; %load image package.
    catch
        error('MovVis:Octave:Image','Image package for Octave not installed'); %display error that image package for Octave is not installe.  
    end
    try
        pkg load video; %load video package
    catch
        error('MovVis:Octave:Video','Video package for Octave not installed'); %display error that video package for Octave is not installe.  
    end
end

%% Construct ID for all category and record combinations and add nan for missing combinations.
if size(xy,2)>6 %if more than one category column included.
    if exist('combvec','file') %if combvec function available.
        combis=eval(['combvec(',sprintf('unique(xy(:,%d))'',',2:size(xy,2)-5),'unique(xy(:,end-4))'');'])'; %all category combinations.
    else
      combis=eval(['combvecOct(',sprintf('unique(xy(:,%d))'',',2:size(xy,2)-5),'unique(xy(:,end-4))'');'])'; %all category combinations.
    end
else
    combis=unique(xy(:,2)); %all category combinations.
end
xy(:,end+1)=0;combis(:,size(combis,2)+1)=0; %preallocate catcombiID column.
for c=(size(combis,2)-1):-1:1 %loop over all category columns.
    xy(:,end)=xy(:,size(xy,2))+xy(:,c+1)*10^(length(num2str(max(xy(:,end))))); %insert unique catcombiID.
    combis(:,end)=combis(:,end)+combis(:,c)*10^(length(num2str(max(combis(:,end))))); %insert unique catcombiID.
end
xy(:,end)=xy(:,end)/10; %correct catcombiID.
combis(:,end)=combis(:,end)/10; %correct catcombiID.
xy(:,end+1)=xy(:,end)*10^(length(num2str(max(xy(:,end-3)))))+xy(:,end-3); %add catcombi x event ID.
combis1=combis; %store first combi.
if exist('combvec','file') %if combvec function available.
    combis=combvec(combis(:,end)',unique(xy(:,end-4))')'; %all catcombi x event combis.
else
    combis=combvecOct(combis(:,end)',unique(xy(:,end-4))')'; %all catcombi x event combis.
    warning('off','Octave:mixed-string-concat'); %suppress warning in Octave.
end
combis(:,end+1)=combis(:,1)*10.^(length(num2str(max(combis(:,2)))))+combis(:,2); %add catcombi x event ID.
[combis,~]=sortrows(combis,3); %sort rows according to catcombi x event ID.
while ~all(unique(combis(:,end))-[unique(xy(:,end));...
        nan(length(unique(combis(:,end)))-length(unique(xy(:,end))),1)]==0) %as long as missing combis in xy.
    ind=find(unique(combis(:,end))-[double(unique(xy(:,end)));...
        nan(length(unique(combis(:,end)))-length(unique(xy(:,end))),1)],1); %find missing combi.
    xy=[xy;nan(1,size(xy,2))]; %add nan row.
    xy(end,[end-1,end-4,end])=combis(ind,:); %add combi info.
    xy(end,2:end-6)=combis1(combis1(:,end)==combis(ind,1),1:end-1); %add combi1 info
    xy(end,[1,end-5])=xy(1,[1,end-5]); %add pseudo timestamp, unit, and event for missing category.
end

%% Align all recorded movements per trace according to desired temporal resolution (speed).
if any(strcmp({'starts','ends','all'},alignment)) %if starts, ends, or all alignment requested.
    combis=unique(xy(:,end-1)); %all category combinations.
    xyH=[];
    for c=1:length(combis) %loop over all category combinations.
        temp=xy(xy(:,end-1)==combis(c),:); %data of combination c.
        temp=temp(:,[1,end-4:end-2]); %timestamp,unit,x,y of all events of combination c.
        temp=HZalign(temp,speed,missub); %align traces according to Hz.
        temp2=[zeros(size(temp,1),1)+c,...
            [temp,nan(size(temp,1),length(unique(xy(:,end-4)))*2+1-size(temp,2))]]; %insert combination index.
        xyH=[xyH;temp2]; %add Hz-algined data of combination c.
    end
    xyT=unique(xyH(:,2)); %needed frames.
    for c=1:max(xyH(:,1)) %loop over all combinations.
        temp=xyH(xyH(:,1)==c,3:end); %all xy of combination c.
        if missub %if missing data shall be substituted.
            temp=[temp;repmat(temp(end,:),size(xyT,1)-size(temp,1),1)]; %keep last xy in final frames.
        else
            temp=[temp;nan(size(xyT,1)-size(temp,1),4)]; %insert NaN in final frames.
        end
        xyT=[xyT,temp]; %add filled data of combination c.
    end
    if nargout==2
        varargout{2}(1)={xyT}; %set Hz-aligned data as first cell in second output call.
    end
end

%% Align all recorded movements per unit according to desired temporal resolution (speed).
if any(strcmp({'units','all'},alignment)) %if units or all alignment requested.
    combis=unique(xy(:,end-1)); %all category combinations.
    units=unique(xy(:,end-5)); %all units.
    xyUH=[];
    for c=1:length(combis) %loop over all category combinations.
        for u=1:length(units) %loop over all units.
            temp=xy(xy(:,end-1)==combis(c)&xy(:,end-5)==units(u),:); % data of combi c in unit u.
            if isempty(temp)
                xyUH=[xyUH;[c,units(u),1,nan(1,length(unique(xy(:,end-4)))*2)]]; %NaN if combination is missing.
            else
                temp=temp(:,[1,end-4:end-2]); %only timestamp, events, and xy columns.
                temp=HZalign(temp,speed,missub); %align event traces according to Hz.
                temp2=[zeros(size(temp,1),1)+c,...
                    zeros(size(temp,1),1)+units(u),...
                    [temp,nan(size(temp,1),length(unique(xy(:,end-4)))*2+1-size(temp,2))]]; %add combi and unit index to Hz-aligned data.
                if u~=1 && missub %if missing data shall be substituted for smooth movement visualization.
                    for m=1:(size(temp2,2)-3)/2 %loop over all events.
                        temp2(isnan(temp2(:,3+m*2-1)),3+m*2-1)=xyUH(end,3+m*2-1); %substitute first missing unit x with last unit x.
                        temp2(isnan(temp2(:,3+m*2)),3+m*2)=xyUH(end,3+m*2); %substitute first missing unit y with last unit y.
                    end
                end
                xyUH=[xyUH;temp2]; %add Hz-aligned data of combi c and unit u.
            end
        end
    end
end

%% Align all recorded movements per unit according to relative duration.
if any(strcmp({'units','all'},alignment)) %if units or all alignment requested.
    xyUR=[];
    for u=1:length(units) %loop over all units.
        temp=xyUH(xyUH(:,2)==units(u),:); %Hz-aligned data of unit u.
        temp(:,2)=[]; %delete unit column.
        xyUR=[xyUR;EndsAlign(temp,missub)]; %stretch all traces of unit u to slowest trace.
    end
    xyUR(:,1)=1:size(xyUR,1); %add frame index.
    if nargout==2
        varargout{2}(3)={xyUR}; %set unit-aligned data as third cell in second output call.
    end
end

%% Align all recorded movements according to relative duration.
if any(strcmp({'ends','all'},alignment)) %if ends or all alignment requested.
    xyR=EndsAlign(xyH,missub); %align records according to relative duration
    if nargout>1
        varargout{2}(2)={xyR}; %set ends-aligned data as second cell in second output call.
    end
end

%% Create video of all record traces.
if any(strcmp({'starts','all'},alignment)) %if starts or all alignment requested.
    videoXY(sprintf('%s',prefix,'_startsAligned'),xyT,picpath,vissize,drawtrace,25,styles); %sample aligned video
end
if any(strcmp({'ends','all'},alignment)) %if ends or all alignment requested.
    videoXY(sprintf('%s',prefix,'_endsAligned'),xyR,picpath,vissize,drawtrace,25,styles); %reldur aligned video
end
if any(strcmp({'units','all'},alignment)) %if units or all alignment requested.
    videoXY(sprintf('%s',prefix,'_unitsAligned'),xyUR,picpath,vissize,drawtrace,25,styles); %unit aligned video
end

end
