%% MovVis_Script
%% MovVis is a movement visualization toolbox.
%% MovVis creates videos and animated gifs with movement animations in desired alignment over a chosen background image.
%
% INSTRUCTIONS:
% INPUT:    REQUIRED:
%           data =     either: full path of file containing the movement data in the required format
%                      or: matlab variable containing the movement data in the required format
%                      with columns: timestamp, categoryID columns 1-n, unitID, eventID, x, y. 
%                      Only positive integers allowed for all inputs.
%                      The movements of all category x event combinations are presented in parallel.
%                      Unit can be used to align movements within separate sub-units,
%                      e.g. until acting on 1. target, until acting on 2. target.
%           OPTIONAL:
%           picpath    = full path of background image - default is empty = grey background.
%                        xy-coordinates are interpreted in pixel on picture, make sure pic has right size.
%                        if specified, visualizations will be saved to visualization sub-folder in the containing folder.
%                        if not specified, visualizations will be saved to visualization sub-folder in current folder.
%           speed      = visualization speed - default is 25.
%                        25 = real time in starts-aligned 25Hz video and gif.
%                        higher values for slow-motion, lower values for fast-forward.
%           vissize    = vector [width,height] defining size of visualization - default is as backgroundimage.
%           alignment  = type of alignment - default is 'sample'.
%                        'starts' = starts aligned, thus normal speed differences included.
%                        'ends' = ends also aligned according to the slowest movement (see MovVis_instructions.txt).
%                        'units' = starts and ends aligned within sub-units according to slowest movement (see MovVis_intructions.txt).
%                        'all' = all three alignments are generated.
%           prefix     = prefix of output file - default is 'MovVis'.
%           drawtrace  = whether to draw trace (1=yes, 0=no) - default is no.
%           missub    = whether to substitute missing data with next data point (1=yes, 0=no) - default is yes.
%           the following styles can be modified:
%           marker     = string of marker styles ('od*s^x>+', see doc plot) - default is all circle.
%           faceCol    = marker face colors (RGB defined by numbers between 0 and 1) - default is 0,0,0 = black.
%           edgeCol    = marker edge colors (RGB defined by numbers between 0 and 1) - default as faceCol.
%           traceCol   = trace colors (RGB defined by numbers between 0 and 1) - default as faceCol.
%           traceLine  = cell of lines styles for the trace (see doc plot) - default is all solid.
%           markerSize = marker sizes - default is 10.
%           !!styles are applied to the data from events over the category columns in decreasing order!!
%           !!example: 4subjects in column2 x 3trials in column3 x 2events in column5:
%           !!maker='od*s+x' mark as follows:
%           !!o for event1 of trial1 of sub1, d for event2 of trial1 of sub1,
%           !!* for event1 of trial2 of sub1, s for event2 of trial2 of sub1,
%           !!+ for event1 of trial3 of sub1, x for event2 of trial3 of sub1,
%           !!o for event1 of trial1 of sub2, d for event2 of trial1 of sub2, and so on
%           !!marker='so' marks each event individually - event1 with 's', event2 with 'o'.
%           !!marker='ssoodd' marks each trial individually - trial1 with 's', trial2 with 'o', trial3 with 'd'.
%           !!marker=[repmat('s',2*3,1),repmat('o',2*3,1),repmat('d',2*3,1)] marks each subject individually in the same order.
%	        !!marker=sort(repmat('sod',2*3,1)) marks each subject individually, but in the reverse order.
%
% OUTPUT:   video and gif file of animated movement paths and picture of end-configuration.
%           OPTIONAL:
%           varargout{1} = matrix array containing the input movement data.
%           varargout{2} = cell array of aligned data matrices 1.starts, 2.ends, 3.units.
%                          column format: frameID, x1, y1, ..., xn, yn
%                          same order as with styles (xy of all events first,
%                          then all events of 2nd category of last category columns, etc.
%
% RMF, 2019

%% Description of example data.
% Example data is described in detail in Foerster, R. M. (2016). Task-irrelevant expectation violations in sequential manual actions: Evidence for a check-after-surprise mode of visual attention and eye-hand decoupling. Frontiers in Psychology, 7, 112. https://doi.org/10.3389/fpsyg.2016.01845
% Example data (mymovedata) contains 100Hz cursor (event 9) and eye (event 23) movements
% recorded from 2 trials (65 & 66) of 10 subjects (1-10)
% while performing a sequential number clicking task (computerized TMT-A).
% A constant number-configuration (ScreenPrechange) was completed for 65 trials (prechange).
% Unannounced in trial 66 (change trial), the font of number 4 changed (ScreenChange).

%% Required input data.
data='E:\MATLAB\MeinMatlab\MovVis\mymovedata.txt'; %path of data file containing movement data in the required format (see above).
% also possible to refer to matlab variable containing data in the required format (see above).

%% Construct videos, gifs, and pictures of trajectories as specified.

%Example1: Default starts-aligned movement visualization.
MovVis(data); %visualization without calling data.

%Example2: Maximally specified units-algined movement visualization.
prefix='example2';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
speed=150; %visualization in slow-motion, both events up-scaled.
vissize=[512,384]; %visualization with half the size of the picture.
alignment='units'; %starts and ends aligned per unit.
drawtrace=1; %permanent trace.
missub=1; %substitute missing data with next point.
traceLine={'-','--'}; %solid for cursor, broken for eye movements.
marker='od'; %circle for cursor, diamant for eye movements.
markerSize=[10,6]; %big for cursor, small for eye movements.
edgeCol=[1 1 1]; %all white.
faceCol=[0 0 1; 0 0 1; 0.6 0.2 0.2; 0.6 0.2 0.2]; %blue for prechange trial, red for change trial.
traceCol=[0.4 0.4 0.8; 0.4 0.4 0.8; 0.4 0.2 0.2; 0.4 0.2 0.2]; %blue for prechange trial, red for change trial.
[xy,xyA]=MovVis(data,prefix,picpath,speed,vissize,alignment,drawtrace,missub,marker,markerSize,traceLine,edgeCol,faceCol,traceCol); %visualization and calling original and aligned data.
clear prefix picpath speed vissize alignment drawtrace missub traceLine marker markerSize edgeCol faceCol traceCol %delete no longer needed values.

%Example3: Starts-aligned movement visualization with selected specifications.
prefix='example3';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
vissize=[512,384]; %visualization with half the size of the picture.
marker='od'; %circle for cursor, diamant for eye movements.
markerSize=[10,6]; %big for cursor, small for eye movements.
faceCol=[0 0 1; 0 0 1; 0.6 0.2 0.2; 0.6 0.2 0.2]; %blue for prechange trial, red for change trial.
xy=MovVis(data,prefix,picpath,vissize,marker,markerSize,faceCol); %visualization and calling only original data.
clear prefix picpath vissize marker markerSize faceCol %delete no longer needed values.

%Example4: Ends-algined movement visualization with individual colors per subject (needs xy of Example3 or code after %).
prefix='example4';
subsCol=rand(length(unique(xy(:,2))),3); %randomly chosen color per subject.
faceCol=sortrows(repmat(subsCol,length(unique(xy(:,3)))*length(unique(xy(:,5))),1)); %give all events and trials the chosen subject color.
edgeCol=sort(repmat([1,1,1;0,0,0],length(unique(xy(:,3))),1)); %black for prechange trial, white for change trial.
%subsCol=rand(10,3); %randomly chosen color per subject.
%faceCol=sortrows(repmat(subsCol,2*2,1)); %give all events and trials the chosen subject color.
%edgeCol=[0,0,0;0,0,0;1,1,1;1,1,1]; %black for prechange trial, white for change trial.
marker='od'; %circle for cursor, diamant for eye movements.
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
vissize=[512,384]; %visualization with half the size of the picture.
alignment='ends'; %starts and ends are algined, unit bounderies ignored.
MovVis(data,prefix,faceCol,edgeCol,marker,picpath,vissize,alignment); %visualization without calling data.
clear subsCol prefix faceCol edgeCol marker picpath vissize alignment %delete no longer needed values.

%Example5: Only prechange trial 65 of all subjects (needs xy of Example3).
data=xy(xy(:,3)==65,:); %only prechange trial 65 data.
prefix='example5';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenPrechange.tif'; %full path of change display picture.
speed=100; %visualization in slow-motion, cursor movements up-scaled.
vissize=[512,384]; %visualization with half the size of the picture.
drawtrace=1; %permanent trace.
traceLine={'-','--'}; %solid for cursor, broken for eye movements.
marker='od'; %circle for cursor, diamant for eye movements.
faceCol=[0 0 0; 0 0.7 0]; %black for cursor, green for eye movements.
edgeCol=[1 1 1]; %all white.
alignment='units'; %starts and ends algined per unit.
[~,xyPA]=MovVis(data,prefix,picpath,speed,vissize,drawtrace,traceLine,marker,faceCol,edgeCol,alignment); %visualization and calling only aligned data.
clear prefix picpath speed vissize drawtrace traceLine marker faceCol edgeCol alignment %delete no longer needed values.

%Example6: Only change trial 66 of all subjects (needs xy of Example3).
data=xy(xy(:,3)==66,:); %only change trial 66 data.
prefix='example6';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
speed=100; %visualization in slow-motion, cursor movements up-scaled.
vissize=[512,384]; %visualization with half the size of the picture.
drawtrace=1; %permanent trace.
traceLine={'-','--'}; %solid for cursor, broken for eye movements.
marker='od'; %circle for cursor, diamant for eye movements.
faceCol=[0 0 0; 0 0.7 0]; %black for cursor, green for eye movements.
edgeCol=[1 1 1]; %all white.
alignment='units'; %starts and ends algined per unit.
[~,xyCA]=MovVis(data,prefix,picpath,speed,vissize,drawtrace,traceLine,marker,faceCol,edgeCol,alignment); %visualization and calling only aligned data.
clear prefix picpath speed vissize drawtrace traceLine marker faceCol edgeCol alignment %delete no longer needed values.

%Example7: Eye and cursor movement of subject 3 in the change trial (needs xy of Example3).
data=xy(xy(:,2)==3&xy(:,3)==66,:); %eye and cursor movement data of subject 3 in trial 66.
prefix='example7';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
vissize=[512,384]; %visualization with half the size of the picture.
drawtrace=1; %permanent trace.
traceLine={'-','--'}; %solid for cursor, broken for eye movement.
marker='od'; %circle for cursor, diamant for eye movement.
faceCol=[0 0 0; 0 0.7 0]; %black for cursor, green for eye movement.
MovVis(data,prefix,picpath,vissize,drawtrace,traceLine,marker,faceCol); %visualization without calling data.
clear prefix picpath vissize drawtrace traceLine marker faceCol %delete no longer needed values.

%Example8: Unit-averaged eye movements of subject 3 (needs xy of Example3).
data=zeros(9*2,7);
for t=1:2 %loop over both trials
    for u=1:9 %loop over all units
        data((t-1)*9+u,:)=nanmean(xy(xy(:,2)==3&xy(:,3)==t+64&xy(:,4)==u&xy(:,5)==23,:)); %mean eye movement data of subject 4 per unit and trial.
    end
end
data(:,1)=[1:9,1:9]*200; %update frame timestamps.
prefix='example8';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
vissize=[512,384]; %visualization with half the size of the picture.
drawtrace=1; %permanent trace.
marker='.'; %points.
faceCol=[0 0 1; 0.6 0.2 0.2]; %blue for prechange trial, red for change trial.
MovVis(data,prefix,picpath,vissize,drawtrace,marker,faceCol); %visualization without calling data.
clear t u prefix picpath vissize drawtrace marker faceCol %delete no longer needed values.

%Example9: Unit-aligned average paths of all subjects per trial and event
%          based on unit-aligned (Example5 and Example6).
data=[xyPA{3}(:,1),repmat([65,9],size(xyPA{3},1),1),...
      mean(xyPA{3}(:,2:4:end),2),mean(xyPA{3}(:,3:4:end),2);... %mean xy cursor positions of prechange trial 65.
      xyPA{3}(:,1),repmat([65,23],size(xyPA{3},1),1),...
      mean(xyPA{3}(:,4:4:end),2),mean(xyPA{3}(:,5:4:end),2);... %mean xy eye positions of prechange trial 65.
      xyCA{3}(:,1),repmat([66,9],size(xyCA{3},1),1),...
      mean(xyCA{3}(:,2:4:end),2),mean(xyCA{3}(:,3:4:end),2);... %mean xy cursor positions of change trial 66.
      xyCA{3}(:,1),repmat([66,23],size(xyCA{3},1),1),...
      mean(xyCA{3}(:,4:4:end),2),mean(xyCA{3}(:,5:4:end),2)]; %mean xy eye positions of change trial 66.
data=round(data(~isnan(data(:,end)),:)); %round and delete nan entries.
data=data([data(1:end-1,end-1)==data(2:end,end-1)&data(1:end-1,end)==data(2:end,end);0]==0,:); %delete identical values.
data=[data(:,1),ones(size(data,1),1),data(:,2),[1:size(data,1)]',data(:,3:end)]; %add pseudo subject, unit.
prefix='example9';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
speed=100; %visualization in slow-motion, cursor movements up-scaled.
vissize=[512,384]; %visualization with half the size of the picture.
drawtrace=1; %permanent trace.
traceLine={'-','--'}; %solid for cursor, broken for eye movements.
marker='od'; %circle for cursor, diamant for eye movements.
faceCol=[0 0 1; 0 0 1; 0.6 0.2 0.2; 0.6 0.2 0.2]; %blue for prechange trial, red for change trial.
MovVis(data,prefix,picpath,speed,vissize,drawtrace,traceLine,marker,faceCol); %visualization without calling data.
clear prefix picpath speed vissize drawtrace traceLine marker faceCol %delete no longer needed values.

%Example10: Successive ovements of all subjects (needs xy of Example3).
subs=unique(xy(:,2)); %all subjects
data=xy;
for n=2:length(subs)
    data(data(:,2)==subs(n),1)=... %timestamps of subject n becomes...
        data(data(:,2)==subs(n),1)... %timestamps of subject n.
        -data(find(data(:,2)==subs(n),1),1)... %-min timestamps of subject n.
        +data(find(data(:,2)==subs(n-1),1,'last'),1)... %+max timestamps of subject n-1.
        +1000; %+1000, so that pause between subjects 1000 ms.
end
data(:,2)=[]; %delete subject index, so that they are no longer presented in parallel, but one after the other.
prefix='example10';
picpath='E:\MATLAB\MeinMatlab\MovVis\ScreenChange.tif'; %full path of change display picture.
vissize=[512,384]; %visualization with half the size of the picture.
marker='od'; %circle for cursor, diamant for eye movements.
markerSize=[10,6]; %big for cursor, small for eye movements.
faceCol=[0 0 1; 0 0 1; 0.6 0.2 0.2; 0.6 0.2 0.2]; %blue for prechange trial, red for change trial.
MovVis(data,prefix,picpath,vissize,marker,markerSize,faceCol); %visualization without calling data.
clear subs n prefix picpath vissize marker markerSize faceCol %delete no longer needed values.

