function videoXY(name,xy,varargin)
%% creates video and gif with 'name' out of 'xy' with Hz and underlied image located at path
%
% INPUT:    name        = name of file to create
%           xy          = data matrix containing movements
%                         with columns: videoframe,x1,y1,x2,y2,...xn,yn
%           varargin{1} = picpath = path of image to underlie (optionally) - default is grey
%           varargin{2} = vissize = vector of video and gif size (optionally) - default is as background image
%           varargin{3} = drawtrace  = 1 for yes and 0 for no (optionally) - default is no
%           varargin{4} = Hz =     Hertz of produced video and gif (optionally) - default is 25
%           varargin{5} = styles = 1x5 cell array with style specifications in the format (optionally)
%                                  1. = string of marker specifications,
%                                       e.g., 'od*s^x>+'
%                                  2. = cell of line specifications,
%                                       e.g., strsplit('- -- : -. - -- : -.')
%                                  3. = matrix of marker face color specifications,
%                                       e.g., [0 0 0;1 0 0;1 1 1;0 1 1;1 1 0;1 0 1;0 0 1;0 1 0]
%                                  4. = matrix of marker edge color specifications,
%                                       e.g., [0 0 0;1 0 0;1 1 1;0 1 1;1 1 0;1 0 1;0 0 1;0 1 0]
%                                  5. = matrix of trace color specifications,
%                                       e.g., [0 0 0;1 0 0;1 1 1;0 1 1;1 1 0;1 0 1;0 0 1;0 1 0]
%                                  6. = vector of marker size specifications,
%                                       e.g., [10,4,3]
%
% RMF, 2019

%% Check and set arguments.
narginchk(2,7); %checks for min and max number of input arguments.
if size(xy,1)==1
    beep; %play beep to signal error.
    error('MovVis:NoMovement','Visualization has only one point. Check data or increase Hz.'); %Abort function and display error.
end
if ~isempty(varargin) && ~isempty(varargin{1})
    picpath=varargin{1}; %set path as defined.
else
    picpath=''; %set empty path = no underlied image.
end
if ~isempty(varargin) && length(varargin)>1 && ~isempty(varargin{2}) && all(varargin{2}>=64) && varargin{2}(1)<=1920 && varargin{2}(2)<=1088
    vissize=[varargin{2}(2),varargin{2}(1)]; %set resize as defined.
else
    vissize=0; %set vissize to 0 = no.
end
if ~isempty(varargin) && length(varargin)>2 && ~isempty(varargin{3})
    drawtrace=varargin{3}; %set trace as defined.
else
    drawtrace=0; %set trace to default = no.
end
if ~isempty(varargin) && length(varargin)>3 && ~isempty(varargin{4})
    Hz=varargin{4}; %set Hz as defined.
else
    Hz=25; %set Hz to default.
end
if ~isempty(varargin) && length(varargin)>4 && ~isempty(varargin{5})
    styles=varargin{5}; %set plot styles to defaults.
else
    styles={'o',[0 0 0],[0 0 0],[0 0 0],'-',10}; %black 10-point circles with solid black traces.
end

%% Check and set styles.
styles{1}=eval(['{',sprintf('styles{1}(%d),',1:length(styles{1})),'};']); %cell of markers.
if drawtrace==1
    stylesNew=num2cell(zeros((size(xy,2)-1)/2,6)); %empty styles cell in the length of all traces.
else
    stylesNew=num2cell(zeros(min([(size(xy,2)-1)/2,...
        lcm(lcm(lcm(lcm(lcm(length(styles{1}),...
        size(styles{2},1)),size(styles{3},1)),size(styles{4},1)),...
        length(styles{5})),length(styles{6}))]),6));
    %empty styles cell in the length of the different style combinations or all traces, if this is less.
end
temp=repmat(styles{1},1,ceil(size(stylesNew,1)/min(size(stylesNew,1),length(styles{1})))); %marker list.
if length(temp)~=size(stylesNew,1)
    warning('MovVis:NoMarkers','number of markers is no aliquot divisor of number of traces'); %marker warning.
end
stylesNew(:,1)=temp(1:size(stylesNew,1)); %add marker list up to number of traces.
temp=repmat(num2cell(styles{2},2)',1,ceil(size(stylesNew,1)/min(size(stylesNew,1),size(styles{2},1)))); %faceCol list.
if length(temp)~=size(stylesNew,1)
    warning('MovVis:NoFaceCol','number of face colors is no aliquot divisor of number of traces'); %facCol warning.
end
stylesNew(:,2)=temp(1:size(stylesNew,1)); %add faceCol list up to number of traces.
temp=repmat(num2cell(styles{3},2)',1,ceil(size(stylesNew,1)/min(size(stylesNew,1),size(styles{3},1)))); %edgeCol list.
if length(temp)~=size(stylesNew,1)
    warning('MovVis:NoEdgeCol','number of edge colors is no aliquot divisor of number of traces'); %edgeCol warning
end
stylesNew(:,3)=temp(1:size(stylesNew,1)); %add edgeCol list up to number of traces.
temp=repmat(num2cell(styles{4},2)',1,ceil(size(stylesNew,1)/min(size(stylesNew,1),size(styles{4},1)))); %traceCol list.
if length(temp)~=size(stylesNew,1)
    warning('MovVis:NoTraceCol','number of trace colors is no aliquot divisor of number of traces'); %traceCol warning
end
stylesNew(:,4)=temp(1:size(stylesNew,1)); %add traceCol list up to number of traces.
temp=repmat(styles{5},1,ceil(size(stylesNew,1)/min(size(stylesNew,1),length(styles{5})))); %traceLine list.
if length(temp)~=size(stylesNew,1)
    warning('MovVis:NoTraceLine','number of trace lines is no aliquot divisor of number of traces'); %traceLine warning
end
stylesNew(:,5)=temp(1:size(stylesNew,1)); %add traceLine list up to number of traces.
temp=repmat(num2cell(styles{6}),1,ceil(size(stylesNew,1)/min(size(stylesNew,1),length(styles{6})))); %add markerSize.
if length(temp)~=size(stylesNew,1)
    warning('MovVis:NoMarkerSize','number of marker sizes is no aliquot divisor of number of traces'); %markerSize warning
end
stylesNew(:,6)=temp(1:size(stylesNew,1)); %add markerSize list up to number of traces.

%% Check and set background image.
if ~isempty(picpath)
    try
        img=imread(picpath); %read background image.
    catch
        picpath=[]; %empty picpath.
        warning('MovVis:picInvalid','picture or its path invalid - grey background used instead'); %warning that picture could not be loaded.
    end
    if size(img,3)>3
        img=img(:,:,1:3); %reduce to three channel image.
    end
end
if isempty(picpath)
    while ~exist('img','var')
        picpath=[pwd,'\']; %set path to current folder
        try
%             img=zeros(ceil(max(max(xy(:,3:2:end))))+floor(min(min(xy(:,3:2:end))))+max(styles{6}),...
%             ceil(max(max(xy(:,2:2:end))))+floor(min(min(xy(:,2:2:end))))+max(styles{6}))+204 %grey background in data range including original zero.
            img=zeros(ceil(max(max(xy(:,3:2:end))))-floor(min(min(xy(:,3:2:end))))+2*max(styles{6}),...
                ceil(max(max(xy(:,2:2:end))))-floor(min(min(xy(:,2:2:end))))+2*max(styles{6}))+204; %grey background in data range + marker size puffer.
            xy(:,2:2:end)=xy(:,2:2:end)-floor(min(min(xy(:,2:2:end))))+max(styles{6}); %and resetting zero in x.
            xy(:,3:2:end)=xy(:,3:2:end)-floor(min(min(xy(:,3:2:end))))+max(styles{6}); %and resetting zero in y.
        catch
            xy(:,2:end)=xy(:,2:end)/max(max(xy(:,2:end)))*1000; %rescale coordinates to max 1000.
            warning('MovVis:invalidDataRange','xy-coordinate range down-sized to 0-1000'); %display warning that data was transformed.
        end
    end
end
if size(img,3)<3
    img=uint8(repmat(img,1,1,3)); %add color channels.
end
if length(vissize)<2 %if only one value is inserted as vissize.
    if size(img,1)<64 || size(img,2)<64
        img=[img;uint8(zeros(64-size(img,1),size(img,2),3))+204]; %add grey area below.
        img=[img,uint8(zeros(size(img,1),64-size(img,2),3))+204]; %add grey area on the right.
        if ~isempty(picpath)
            warning('MovVis:picTooSmall','picture was too small - grey area added'); %warning that grey area was added.
        end
    end
    if size(img,1)>1088 || size(img,2)>1920
        xy(:,2:2:end)=xy(:,2:2:end)*(1*(size(img,2)<=1920)+(size(img,2)>1920)/size(img,2)*1920); %rescale x accordingly, if required.
        xy(:,3:2:end)=xy(:,3:2:end)*(1*(size(img,1)<=1088)+(size(img,1)>1088)/size(img,1)*1088); %rescale y accordingly, if required.
        img=imresize(img,[min(size(img,1),1088),min(size(img,2),1920)]); %resize picture to acceptable range.
        warning('MovVis:visTooBig','visualization out of range - downscaled'); %warning that data was downscaled. 
    end
end
if size(img,1)<max(max(xy(:,3:2:end)))
%         img=[img;repmat(img(end,end,end),abs(max(max(xy(:,3:2:end)))-size(img,1)),size(img,2),3)]; %add area below matching last pixel's color.
%         img=imresize(img,[max(max(xy(:,3:2:end))),size(img,2)]); %stretch height to match data - coordinates do no longer match.
    warning('MovVis:dataOutOfPicHeigth','data out of range of picture height'); %display warning that data out of range of picture heigth.
end
if size(img,2)<max(max(xy(:,2:2:end)))
%         img=[img,repmat(img(end,end,end),size(img,1),abs(max(max(xy(:,2:2:end)))-size(img,2)),3)]; %add area rightwards matching last pixel's color.
%         img=imresize(img,[size(img,1),max(max(xy(:,2:2:end)))]); %stretch width to match data - coordinates do no longer match.
    warning('MovVis:dataOutOfPicWidth','data out of range of picture width'); %display warning that data out of range of picture width.
end
if mod(size(img,1),2)~=0
    img=[img;img(end,:,:)]; %double last line to get to even heigth.
    warning('MovVis:oddPicHeigth','last picture line doubled to get even height.'); %display warning that last picture line was doubled.
elseif mod(size(img,2),2)~=0
    img=[img,img(:,end,:)]; %double last row to get even width.
    warning('MovVis:oddPicWidth','last picture row doubld to get even width'); %display warning that last picture row was doubled.
end
if length(vissize)<2 %if only one value is inserted as vissize.
    vissize=[size(img,1),size(img,2)]; %set visualization size to background image.
end

%% Plot and capture.
temp=[strfind(picpath,'\'),strfind(picpath,'/')]; %indices of all slashes in path.
if isempty(temp)
    temp=0; %no slashes in path.
end
if ~isempty(strfind(version,'R')) %if Matlab.
    warning('off','MATLAB:print:DeprecateZbuffer'); %disable warning for hardcopy.
    warning('off','MATLAB:hardcopy:DeprecatedHardcopyFunction'); %disable warning for hardcopy.
end
fig=figure('visible','off'); %open figure.
set(fig,'Position',[0,0,size(img,2),size(img,1)]); %adjust position on screen.
image(img); hold on; %display background image.
axis([0,size(img,2),0,size(img,1)],'ij','manual'); %adjust and fix axis.
set(gca,'XTick',[],'YTick',[],'xcolor','k','ycolor','k','box','on'); %black box without tick marks for cropping.
% set(gca,'XAxisLocation','top',...
%     'XTick',[0,size(img,2)/2,size(img,2)],...
%     'YTick',[0,size(img,1)/2,size(img,1)]); %adjust visible axis (use this and comment cropping lines out, if axis shall be viewed).
try
    gif=hardcopy(fig,'-dzbuffer','-r0'); %image of current figure.
catch
    gif=frame2im(getframe(fig)); %image of current figure with showing.
end
if exist('combvec','file') %if executed in Matlab
  [r,c]=find(gif(:,:,1)==0,1,'first'); %row and column index of first corner.
  [rr,cc]=find(gif(:,:,1)==0,1,'last'); %row and column index of last corner.
else %if executed in Octave
  [r,c]=find(gif(:,:,1)<50,1,'first'); %row and column index of first corner.
  [rr,cc]=find(gif(:,:,1)<50,1,'last'); %row and column index of last corner.
end
if ~exist(sprintf('%s',picpath(1:temp(end)),'visualizations'),'dir')
    mkdir(sprintf('%s',picpath(1:temp(end)),'visualizations')); %create visualization folder.
end
if exist('VideoWriter','file')
  vid=VideoWriter(sprintf('%s',picpath(1:temp(end)),'visualizations\',name),'MPEG-4'); %create video file.
  vid.FrameRate=Hz; %set frame rate to 25.
  % vid.Quality=20; %set quality to 25.
  open(vid); %open video file
else
    vid=avifile(sprintf('%s',picpath(1:temp(end)),'visualizations\',name,'.mp4'),...
        'fps',Hz); %create video file.
end
p=zeros(size(xy,1),(size(xy,2)-1)/2); %initialize point handle.
for n=1:size(xy,1)-1 %loop over all rows.
    for m=1:size(stylesNew,1) %loop over all traces
        if drawtrace==1 %if trace shall be drawn
            xInd=1+(m-1)*2+1;pInd=m;
        else
            xInd=2*m:2*size(stylesNew,1):size(xy,2);pInd=length(xInd)*(m-1)+1:length(xInd)*m;
        end
        p(n,pInd)=plot(xy(n,xInd),xy(n,xInd+1),'LineStyle','none',...
            'Marker',stylesNew{m,1},'MarkerSize',stylesNew{m,6},...
            'MarkerEdgeColor',stylesNew{m,3},'MarkerFaceColor',stylesNew{m,2}); %plot current point.
        if n~=1 && drawtrace==1 %if not first point of event and trace shall be drawn.
            plot(xy([n-1,n],1+(m-1)*2+1),xy([n-1,n],1+(m-1)*2+2),...
                'LineStyle',stylesNew{m,5},'Color',stylesNew{m,4});
                %plot trace from last to current point of movement.
        end
    end
    if n~=1
        delete(p(n-1,:)); %delete last points.
    end
    try
        gif=hardcopy(fig,'-dzbuffer','-r0'); %image of current figure.
    catch
        gif=frame2im(getframe(fig)); %image of current figure with showing.
    end
    gif=gif(r+1:rr-1,c+1:cc-1,:); %crop plot.
    gif=imresize(gif,vissize); %resize gif.
    imgs(:,:,:,n)=gif; %add gif to multi-image array.
    if exist('VideoWriter','file')
        writeVideo(vid,im2frame(gif)); %add current figure as videoframe to video without showing the plot.
    else
        addframe(vid,gif/255); %add current figure as videoframe to video.
        %addframe(vid,im2frame(gif)); %add current figure as videoframe to video.
    end
    try
        [imind,cm]=rgb2ind(gif,256);
    catch
        [imind,cm]=rgb2ind(gif);
    end
    if ~isempty(strfind(version,'R')) %if Matlab.
        if n==1      
            imwrite(imind,cm,sprintf('%s',picpath(1:temp(end)),'visualizations\',name,'.gif'),...
                'gif','DelayTime',1000/Hz/1000,'Loopcount',inf); %write first pic to animated gif.
        else
            imwrite(imind,cm,sprintf('%s',picpath(1:temp(end)),'visualizations\',name,'.gif'),...
                'gif','DelayTime',1000/Hz/1000,'WriteMode','append'); %write next pic to animated gif.
        end
    end
    if n~=1
        fprintf(repmat('\b',1,18+length(num2str(round((n-1)/size(xy,1)*100))))); %delete last progress info.
    end
    fprintf([num2str(round(n/size(xy,1)*100)),' percent completed']); %display progress.
end
close(fig); %closes figure.
close(vid); %closes video file.
if isempty(strfind(version,'R')) %if Octave.
    imwrite(imgs,sprintf('%s',picpath(1:temp(end)),'visualizations\',name,'.gif'),...
        'gif','DelayTime',1000/Hz/1000,'Loopcount',Inf,'Compression','deflate'); %write all pics to animated gif.
end
imwrite(gif,sprintf('%s',picpath(1:temp(end)),'visualizations\',name,'.tif'),'tif','Compression','lzw'); %save end configuration as picture.
fprintf(repmat('\b',1,18+length(num2str(round((n)/size(xy,1)*100))))); %delete last progress info.
fprintf('100 percent completed\n'); %show end.
warning('on','all'); %enable all warnings.
if exist('commandwindow','file')
  commandwindow; %bring Matlab window to front to signal end.
else
  beep; %play beep to signal end in octave.
end

end